.module Entity_Manager

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;; INCLUDES             ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "entity.inc"
.include "cpctelera.inc"
.include "sprites.inc"
.include "IA.inc"
.include "Animaciones.inc"

.globl sys_collision_update_entities

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Variables Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Variables Iniciales ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;// NUMERO DE ENTIDADES
m_num_entities:: .db #0
;// TAMANO ARRAY DE ENTIDADES
_sizeArray == MAX_ENTITIES * ENTITY_SIZE
;// ARRAY DE ENTIDADES
_m_entities::
    .ds _sizeArray
m_zero_type_at_the_end:: .db #0
;// DIRECCION DE MEMORIA QUE APUNTA
;// A LA SIGUIENTE POSICION LIBRE DEL ARRAY
m_next_free_entity:: .dw #_m_entities

forall_filter: .db #0
forall_function: .dw #0x0000
bullet_entities:: .ds _sizeArray
soldier_entities:: .ds _sizeArray

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'MAN_ENTITY_CREATE'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Crea una nueva entidad de tipo DEFAULT
;; Devuelve en 'DE' la direccion de la entidad creada
;;
;; Destruye HL, DE, BC, A
;;
man_entity_create::
    ld a, (m_num_entities)     
    cp #MAX_ENTITIES
    jp z, saltar_creacion

    ld  hl, #e_type_invalid             
    ld  de, (m_next_free_entity)
    ld  bc, #ENTITY_SIZE
    ldir                                    ; Ponemos a 0 el espacio de la siguiente entidad

    ld  a, (m_num_entities)                 ; Incrementamos en 1 el numero de entidades actuales
    inc a
    ld  (m_num_entities), a

    ld  hl, #ENTITY_SIZE
    ld  de, (m_next_free_entity)
    push ix
    push de
    add hl, de                              ; Apuntamos a la siguiente entidad libre
    ld  (m_next_free_entity), hl            ; Y la marcamos
    pop ix
    ld Ent_TYPE(ix), #e_type_default        ; Ponemos el tipo de la entidad creada a 0xFF (default)
    ld Ent_COMPONENTS(ix), #e_cmp_default   ; POnemos la componente de la entidad creada a 0x00
    pop ix

    saltar_creacion:
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_For_All_Matching'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (FORALL_FUNCTION) Puntero a la funcion a llamar
;;        (FORALL_FILTER) Componente a buscar en las entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Busca las entidades que tengan el componente
;; indicado (D) y llama a la funcion que se pasa en
;; (HL) con dichas entidades
;;
;; DESTRUYE DE, BC, A, IX
;;
man_entity_forall_matching::
    ld ix, #_m_entities                             ; Empezamos a buscar desde la primera entidad
    bucle_man_entity_forall_matching:               ; del array
        ld a,Ent_TYPE(ix)
        cp #e_type_invalid                          ; Pararemos el bucle cuando una entidad sea de tipo
        jp z, saltar_man_entity_forall_matching     ; inválida
            ld a,(forall_filter)
            ld d,a
            ld a,Ent_COMPONENTS(ix)
            and d                                   ; Comprobamos que una de las componentes de la 
            cp d                                    ; entidad, sea la que buscamos
            jp nz, saltar_funcion_matching
                ld bc, #return_man_entity_matching
                push bc
                ld hl, (forall_function)
                jp (hl)                             ; Llamamos a la funcion que se nos dice en (HL)
                return_man_entity_matching::
                saltar_funcion_matching:
                ld de, #ENTITY_SIZE                  ; Pasamos el puntero que utilizamos para recorrer
                add ix, de                          ; el array a la siguiente entidad
        jp bucle_man_entity_forall_matching
    saltar_man_entity_forall_matching:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_For_Pairs_Matching'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero a funcion a llamar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Busca dos entidades en el array que
;; tengan el componente 'COLLIDER' y envía
;; las dos entidades a (HL) en los
;; registros (IX) y (DE)
;;
;; DESTRUYE DE, BC, IX, A
;;
man_entity_forall_pairs_matching::
    ld ix, #_m_entities                                     ;; IX posicion de la primera entidad a comprobar
    bucle_man_entity_forall_pairs_matching:
        ld a,Ent_TYPE(ix)                                          ;; Guardamos su tipo
        cp #e_type_invalid                                  ;; Comprobamos que no es invalida
        jp z, saltar_man_entity_forall_pairs_matching
            ld  a,Ent_COMPONENTS(ix)                                    ;; Guradamos su componente
            and #e_cmp_collider                             ;; Comprobamos que la componente es la indicada
            cp  #e_cmp_collider
            jp nz, saltar_funcion_matching_pairs
                ld iy, #_m_entities                
                bucle_segunda_entidad:
                    ld a, Ent_TYPE(iy)                                         ;; Guardamos el tipo de la siguiente entidad
                    cp #e_type_invalid                                  ;; Comprobamos que no es invalida
                    jp z, saltar_bucle_segunda_entidad
                        ld  a,Ent_COMPONENTS(iy)                                    ;; Guradamos su componente
                        and #e_cmp_collider                             ;; Comprobamos que la componente es la indicada
                        cp  #e_cmp_collider
                        jp nz, saltar_funcion_matching_pairs_2component
                        ld  a, Ent_COLISION(ix)                                   ;; Guradamos el componente de la siguiente entidad
                        cp  Ent_TYPE(iy)
                        jp nz, saltar_funcion_matching_pairs_2component
                            ld bc, #return_man_entity_pairs_matching
                            push bc
                            ld hl, #sys_collision_update_entities ;;NUEVO
                            jp (hl)
                            return_man_entity_pairs_matching::
                        saltar_funcion_matching_pairs_2component:
                        ld   de, #ENTITY_SIZE
                        add iy, de                          ;; Pasamos a la siguiente entidad
                    jp bucle_segunda_entidad   
                saltar_bucle_segunda_entidad: 
                saltar_funcion_matching_pairs:
                ld de, #ENTITY_SIZE
                add ix, de                                  ;; Pasamos a la siguiente entidad
        jp bucle_man_entity_forall_pairs_matching
    saltar_man_entity_forall_pairs_matching:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_For_All'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero funcion a llamar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recorre todo el array de entidades
;; y llama a la funcion en (HL) pasando
;; la entidad en (IX)
;;
;; DESTRUYE DE, BC, A, IX
;;
man_entity_forall::
    ld ix, #_m_entities
    bucle_man_entity_forall:
        ld a,Ent_TYPE(ix)
        cp #e_type_invalid                  ; Si una entidad es invalida se sale del bucle
        jp z, saltar_man_entity_forall
            push hl
            ld bc, #return_man_entity       ; Metemos en la pila la direccion de memmoria a la que saltara
            push bc                         ; la función (HL) al llamar al return
            jp (hl)                         ; Llamamos a la funcion que nos pasan por parámetro
            return_man_entity::
            pop hl
            push de
            ld de, #ENTITY_SIZE             ; Apuntamos a la siguiente entidad del array
            add ix, de
            pop de
        jp bucle_man_entity_forall
    saltar_man_entity_forall:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_Destroy'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Puntero a la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; DESTRUYE HL, BC, DE, A, IX
;;
man_entity_destroy::
    ld hl, (m_next_free_entity)

    ld de, #ENTITY_SIZE
    ld a, d
    cpl
    ld d, a
    ld a, e
    cpl
    ld e, a
    inc de
    add hl, de
    push ix
    pop de
    ld a,d
    cp h
    jp nz, saltar_man_entity_destroy
        ld a,e
        cp l
        jp z, coincide_man_entity_destroy
    saltar_man_entity_destroy::
        push ix
        pop de
        ld bc, #ENTITY_SIZE
        push hl
        call cpct_memcpy_asm
        pop hl
    coincide_man_entity_destroy::
    ld (m_next_free_entity), hl         
    ld (hl), #e_type_invalid


    ld a, (m_num_entities)              ; Decrementamos en 1 el numero de entidades actuales
    dec a
    ld (m_num_entities), a 
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_Set4destruction'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Puntero a la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Marca la entidad en (IX) como tipo
;; 'dead' para que en el siguiente update
;; de entidades lo destruya
;;
;; DESTRUYE A
;;
man_entity_set4destruction::
    ld a, Ent_TYPE(ix)
    or #e_type_dead
    ld Ent_TYPE(ix), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_Update'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recorre todo el array de entidades
;; actualizando su tipo para destruirlos
;;
;; Destruye DE, IX, A
;;
man_entity_update::
    ld ix, #_m_entities
    bucle_man_entity_update::
        ld a,Ent_TYPE(ix)
        cp #e_type_invalid                          ; Si una entidad es invalida se sale del bucle
        jp z, saltar_man_entity_update
            and #e_type_dead
            cp #e_type_dead                         ; funcion 'man_entity_destroy' para liberar su espacio
            jp nz, saltar_bucle_man_entity_update
                call man_entity_destroy
                jp bucle_man_entity_update
            saltar_bucle_man_entity_update::
            ld de, #ENTITY_SIZE
            add ix, de
        jp bucle_man_entity_update
    saltar_man_entity_update::

    ld ix, (direccion_jugador)
    ld  a, Ent_COOLDOWN(ix)
    cp  #0
    jp  z, ya_dispara
        dec a
        ld Ent_COOLDOWN(ix), a
    ya_dispara:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_FreeSpace'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Devuelve el número de entidades 
;; libres en el registro (A)
;;
;; Destruye A, E
;;
man_entity_freeSpace::
    ld a, (m_num_entities)           ; Guardamos el número de entidades actuales
    ld e, #MAX_ENTITIES              ; Guardamos el total de entidades que podemos tener
    sub a, e                         ; Restamos ambos valores y el resto se queda en (A)
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetSprite'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;        (HL) Direccion de Sprite
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia el sprite de la entidad
;;
man_entity_setSprite::

    ld Ent_SPRITE1(ix), l
    ld Ent_SPRITE2(ix), h

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetPosition'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;         (H) Posicion en X
;;         (L) Posicion en Y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia la posicion en X e Y
;;
man_entity_setPosition::

    ld Ent_X(ix), h
    ld Ent_Y(ix), l

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetIA'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;        (HL) Tipo de IA
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia el tipo de IA de la entidad
;;
man_entity_setIA::

    ld Ent_AI1(ix), l
    ld Ent_AI2(ix), h

    ld a, #<sys_ia_sniper
    cp l
    jp nz, mantenemos_ia
    ld a, #>sys_ia_sniper
    cp h
    jp nz, mantenemos_ia
        ld Ent_AIPREV1(ix), #<sys_ia_sniper_en_posicion
        ld Ent_AIPREV2(ix), #>sys_ia_sniper_en_posicion
        jp fin_man_entity_setIA 

    mantenemos_ia:

    ld Ent_AIPREV1(ix), l
    ld Ent_AIPREV2(ix), h

    ld a, #<sys_ia_seguir_jugador
    cp l
    jp nz, fin_ia_seguir_jugador
    ld a, #>sys_ia_seguir_jugador
    cp h
    jp nz, fin_ia_seguir_jugador
        ld Ent_COLISION(ix), #e_type_player
        ;arreglo coyote?
        ld Ent_ANIM1(ix), #<anim_coyote_down
        ld Ent_ANIM2(ix), #>anim_coyote_down
        ld Ent_W(ix), #SPR_COYOTE_W
        ld Ent_H(ix), #SPR_COYOTE_H
        jp fin_man_entity_setIA

    fin_ia_seguir_jugador:

    ld a, #<sys_ia_bouncer
    cp l
    jp nz, fin_ia_bouncer
    ld a, #>sys_ia_bouncer
    cp h
    jp nz, fin_ia_bouncer
        ;ld Ent_VX(ix), #VELOCIDAD_IA_BOUNCER_STROLL
        ld Ent_VY(ix), #VELOCIDAD_IA_BOUNCER_STROLL
        ld Ent_COLISION(ix), #e_type_player
        ld Ent_W(ix), #SPR_CALABAZA_W
        ld Ent_H(ix), #SPR_CALABAZA_H
        ld Ent_ANIM1(ix), #<anim_calabaza_right
        ld Ent_ANIM2(ix), #>anim_calabaza_right
        jp fin_man_entity_setIA

    fin_ia_bouncer:

    ld a, #<sys_ia_boss
    cp l
    jp nz, fin_ia_boss
    ld a, #>sys_ia_boss
    cp h
    jp nz, fin_ia_boss
        ld Ent_AIPREV1(ix), #<sys_ia_boss_en_posicion
        ld Ent_AIPREV2(ix), #>sys_ia_boss_en_posicion
        jp fin_man_entity_setIA
    fin_ia_boss:

    fin_man_entity_setIA:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetCooldown'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;         (A) Valor del Cooldown
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia el cooldown de IA de la entidad
;;
man_entity_setCooldown::

    ld Ent_AI_COUNT(ix), a
    ld Ent_AI_COUNT_O(ix), a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetCooldown_SHOT'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;         (A) Valor del Cooldown
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia el cooldown de IA de la entidad
;;
man_entity_setCooldown_shot::

    ld Ent_COOLDOWN(ix), a
    ld Ent_COOLDOWN_O(ix), a

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'Man_Entity_SetPatrol'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad enemigo
;;        (HL) Direccion de memoria de la patrulla
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Cambia el patrulla de IA de la entidad
;;
man_entity_setPatrol::

    ld Ent_PATROL1(ix), l
    ld Ent_PATROL2(ix), h

    ret