.module HUD_Manager

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;      INCLUDES        ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.inc"
.include "HUD.inc"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl show_defeat_screen

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Variables Iniciales ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

m_hud::
    .ds HUD_SIZE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;    PLANTILLAS     ;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
init_HUD::
    .db #4                  ; Número de vidas al comienzo de la partida 0
    .db #1                  ; Nivel de la partida                       1
    .db #1                  ; Escenario de la partida                   2
    .db #20                 ; Final del nivel                           3
    .db #0                  ; Puntuacion    Diez miles                  4
    .db #0                  ; Puntuacion    Miles                       5
    .db #0                  ; Puntuacion    Centenas                    6
    .db #0                  ; Puntuacion    Decenas                     7
    .db #0                  ; Puntuacion    Unidades                    8
    .db #20                 ; Finl puntos                           


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_init'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Incicializa a valores originales
;; el HUD
;;
man_HUD_init::
    ld   hl, #init_HUD                      ; Cogemos la direccion de memoria de la plantilla
    call man_HUD_template                   ; Creamos el HUD
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_teamplate'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Direccion de memoria
;;        de la plantilla a copiar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Incicializa a valores originales
;; el HUD
;;
man_HUD_template:
    ld de, #m_hud
    ld bc, #HUD_SIZE
    call cpct_memcpy_asm    ;(2B DE) to	Pointer to the destination (first byte where bytes will be written)
                            ;(2B HL) from	Pointer to the source (first byte from which bytes will be read)
                            ;(2B BC) size	Number of bytes to be set (>= 1)
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;; FUNCIONES PARA AÑADIR VALORES ;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_life'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al número de vidas del jugador 
;;
man_HUD_add_life::
    ld   hl, #m_hud                         ; Cogemos el valor de las vidas

    ld    a, (hl)
    cp    #9
    jp    nz, se_puede_sumar
    ld    a, b
    cp    #1
    jp    z,  no_se_puede_sumar
        se_puede_sumar:
        ld    a, (hl)
        add   a, b                          ; Le sumamso el valor del registro B
        ld (hl), a                          ; Lo devolvemos al 'array'
    no_se_puede_sumar:

    ; Comprueba si a es 0
    or a            ; Esto pone el flag Z si a es 0
    jp z, show_defeat_screen ; Salta a show_defeat_screen si a es 0
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_next_level'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al número del nivel 
;;
man_HUD_next_level::
    ld   ix, #m_hud                     ; Cogemos el valor del nivel
    ld    a, 1(ix)
    add   a, b

    cp   #6                            ; Le restamos diez
    jp    c, fin_man_HUD_next_level     ; Si no tiene carry acabamos la funcion
        sub  a, #5
        push af
        ld   a, 2(ix)
        inc  a
        ld  2(ix), a
        pop af
    fin_man_HUD_next_level:
    ld 1(ix), a                          ; Lo devolvemos al array
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_diezmil'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al valor correspondiente
;; a los valores x10 000
;;
man_HUD_add_diezmil::
    ld    ix, #m_hud
    ld     a, 4(ix)
    add    a, b
    ld 4(ix), a 
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_mil'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al valor correspondiente
;; a los valores x1 000
;;
man_HUD_add_mil::
    ld   ix, #m_hud
    ld    a, 5(ix)
    add   a, b

    cp   #10                            ; Le restamos diez
    jp    c, fin_man_HUD_add_mil        ; Si no tiene carray acabamos la funcion
        
        sub  a, #10                     ; Restamos esos diez de las unidades actuales
        push af
        ld   b, #1                      ; Si no tiene carray hay que sumar uno a la siguiente unidad
        call man_HUD_add_diezmil
        pop  af

    fin_man_HUD_add_mil:
    ld 5(ix), a 
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_cien'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al valor correspondiente
;; a los valores x100
;;
man_HUD_add_cien::
    ld   ix, #m_hud
    ld    a, 6(ix)
    add   a, b

    cp   #10                            ; Le restamos diez
    jp    c, fin_man_HUD_add_cien       ; Si no tiene carray acabamos la funcion
        
        sub  a, #10                     ; Restamos esos diez de las unidades actuales
        push af
        ld   b, #1                      ; Si no tiene carray hay que sumar uno a la siguiente unidad
        call man_HUD_add_mil
        pop  af

    fin_man_HUD_add_cien:
    ld 6(ix), a 
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_diez'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al valor correspondiente
;; a los valores x10
;;
man_HUD_add_diez::
    ld   ix, #m_hud
    ld    a, 7(ix)
    add   a, b

    cp   #10                            ; Le restamos diez
    jp    c, fin_man_HUD_add_diez       ; Si tiene carray acabamos la funcion
        
        sub  a, #10                     ; Restamos esos diez de las unidades actuales
        push af
        ld   b, #1                      ; Si no tiene carray hay que sumar uno a la siguiente unidad
        call man_HUD_add_cien
        pop  af

    fin_man_HUD_add_diez:
    ld 7(ix), a 
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_add_unidad'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:  (B) Valor a sumar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Suma el valor que se le pasa por 
;; parámetro al valor correspondiente
;; a los valores x1
;;
man_HUD_add_unidad::
    ld   ix, #m_hud
    ld    a, 8(ix)
    add   a, b

    cp   #10                            ; Le restamos diez
    jp    c, fin_man_HUD_add_unidad     ; Si tiene carray acabamos la funcion
        
        sub  a, #10                     ; Restamos esos diez de las unidades actuales
        push af
        ld   b, #1                      ; Si no tiene carray hay que sumar uno a la siguiente unidad
        call man_HUD_add_diez
        pop  af
        

    fin_man_HUD_add_unidad:
    ld 8(ix), a 
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;  FUNCIONES PARA EL RENDER  ;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_forall'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero a funcion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recorre todos los valores  
;; del HUD y los envia al render 
;; para pintarlos en las posiciones 
;; que nosotros indiquemos
;;
man_HUD_forall::
    ld   b, #188
    ld    c, #59
    call man_HUD_forall_score
    ld   b, #188
    ld    c, #11                            ; Posicion en X 
    call man_HUD_forall_level
    ld    b, #188                           ; Posicion en Y
    ld    c, #33   
    call man_HUD_forall_life
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_forall_life'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero a funcion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Coge el valor de las vidas alacenado
;; en el HUD y lo envia al render para
;; ser dibujado en pantalla
;;
man_HUD_forall_life::

    ld   d, b                               ; Guarda b en d
    ld   e, c                               ; Guarda c en e

    ld   ix, #m_hud                         ; Coge el valor del nivel
    ld   a, (ix)       
    call man_HUD_what_number                ; Y lo pasamos al método que devuelve su string en el registro IY            

    ld   bc, #return_man_HUD_forall_life    ; Guardamos el valor del retorno del salto
    push bc                                 ; La colocamos en la pila

    ; Restaura los valores originales en B y C desde D y E
    ld   b, d                               
    ld   c, e                               

    jp   (hl)                               ; Saltamos al puntero a función
    
    ; Aquí, después del salto, B y C todavía tienen los valores originales.
    return_man_HUD_forall_life:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_forall_level'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero a funcion
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Coge el valor del nivel almacenado
;; en el HUD y lo envia al render para
;; ser dibujado en pantalla
;;
man_HUD_forall_level::

    ld   ix, #m_hud     
    inc  ix   

    inicio_man_HUD_forall_level:
    ld    a, (ix)                          ; Cogemos el valor del nivel
    cp   #20
    jp   z, fin_del_bucle_man_HUD_forall_level
    call man_HUD_what_number                ; Y lo pasamos al meétodo que devuelve su string
                                            ; en el registro IY                                                   

    ld d,b
    ld e,c

    ld   bc, #return_man_HUD_forall_level   ; Guardamos el valor del retorno del salto
    push hl
    push bc

    ld b,d
    ld c,e

    jp   (hl)                               ; Saltamos al puntero a funcion
    
    return_man_HUD_forall_level:
    pop  hl
    inc  ix
    inc   c
    inc   c
    inc   c
    inc   c
    inc   c
    inc   c
    inc   c
    inc   c
    jp   inicio_man_HUD_forall_level

    fin_del_bucle_man_HUD_forall_level:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_forall_score'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (HL) Puntero a funcion
;;        ( C) Posicion en X
;;        ( B) Posicion en Y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recorre todos los valores del score 
;; del HUD y los envia al render 
;; para pintarlos en las posiciones 
;; que nosotros indiquemos
;;
man_HUD_forall_score::

    ld   ix, #m_hud                     ; Valor incial del 'array' del HUD
    inc  ix
    inc  ix
    inc  ix
    inc  ix
inicio_bucle_man_HUD_forall_score:

    ld   a, (ix)                        ; Metemos en A el valor
    cp   #20
    jp   z, fin_del_bucle_man_HUD_forall_score
    call man_HUD_what_number            ; Devuelve en IY la direccion de memoria de la cadena a dibujar

    push bc
    pop  de

    ld   bc, #return_man_HUD_forall_score     ; Guardamos en la pila el valor de retorno
    push hl
    push bc
    
    push de
    pop  bc
    jp   (hl)
    return_man_HUD_forall_score:              ; Llamamos al render para que nos pinte el valor que le pasamos
    pop  hl

    inc  ix                             ; Pasamos al siguiente valor del array del HUD
    inc   c
    inc   c
    inc   c
    inc   c
    jp   inicio_bucle_man_HUD_forall_score

    fin_del_bucle_man_HUD_forall_score:       ; Fin del bucle
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'man_HUD_forall'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (A) Valor a convertir
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Convierte el valor que se le pase por 
;; parámetro a una cadena de texto que 
;; devuelve en IY
;;
string0: .asciz "0"
string1: .asciz "1"
string2: .asciz "2"
string3: .asciz "3"
string4: .asciz "4"
string5: .asciz "5"
string6: .asciz "6"
string7: .asciz "7"
string8: .asciz "8"
string9: .asciz "9"
man_HUD_what_number::
    cp  #0
    jp  z, es_cero

    cp  #1
    jp  z, es_uno

    cp  #2
    jp  z, es_dos

    cp  #3
    jp  z, es_tres

    cp  #4
    jp  z, es_cuatro

    cp  #5
    jp  z, es_cinco

    cp  #6  
    jp  z, es_seis

    cp  #7
    jp  z, es_siete

    cp  #8
    jp  z, es_ocho

    cp  #9
    jp  z, es_nueve

es_cero:
    ld iy, #string0
    jp fin_man_HUD_what_number
es_uno:
    ld iy, #string1
    jp fin_man_HUD_what_number
es_dos:
    ld iy, #string2
    jp fin_man_HUD_what_number
es_tres:
    ld iy, #string3
    jp fin_man_HUD_what_number
es_cuatro:
    ld iy, #string4
    jp fin_man_HUD_what_number
es_cinco:
    ld iy, #string5
    jp fin_man_HUD_what_number
es_seis:
    ld iy, #string6
    jp fin_man_HUD_what_number
es_siete:
    ld iy, #string7
    jp fin_man_HUD_what_number
es_ocho:
    ld iy, #string8
    jp fin_man_HUD_what_number
es_nueve:
    ld iy, #string9

fin_man_HUD_what_number:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'reset_HUD'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Reinicio el HUD a sus valores iniciales 
;;
reset_HUD::
    ld hl, #init_HUD        ; apunta a los valores iniciales del HUD
    ld de, #m_hud           ; apunta al inicio del HUD en la memoria
    ld bc, #HUD_SIZE        ; 8 bytes de datos para copiar  
    ldir                    ; copia los valores iniciales al HUD
ret
