.module Physics_System

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;      INCLUDES        ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.include "cpctelera.inc"
.include "cpctelera.h.s"
.include "entity.inc"
.include "sprites.inc"
.include "Animaciones.inc"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl man_game_player_shot
.globl man_game_entity_destroy
.globl sys_music_mute
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Physics_Check_Keyboard'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad a mover
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Actualiza la posicion de la entidad 
;; segun input.Solo funciona para la nave
;; Tenemos que mantener el registro
;; (DE) ya que es donde se encuentra el 
;; componente que se busca en el bucle matching
;;

sys_physics_check_keyboard::
    push de                         ; Mantenemos el registro 'DE'

    ld Ent_VX(ix), #0              
    ld Ent_VY(ix), #0               ; Si no se ha pulsado ninguna tecla sus velocidades son nulas       

    ld hl, #Key_O                   ; Si es la tecla 'A' se mueve a la izquierda
    call cpct_isKeyPressed_asm
    jp nz, input_izquierda
    ld hl, #Joy0_Left                  
    call cpct_isKeyPressed_asm
    jp nz, input_izquierda
    fin_izquierda:

    ld hl, #Key_P                   ; Si es la tecla 'D' se mueve a la derecha
    call cpct_isKeyPressed_asm
    jp nz, input_derecha
    ld hl, #Joy0_Right                 
    call cpct_isKeyPressed_asm
    jp nz, input_derecha
    fin_derecha:

    ld hl, #Key_Q                   ; Si es la tecla 'W' se mueve arriba
    call cpct_isKeyPressed_asm
    jp nz, input_arriba
    ld hl, #Joy0_Up                  
    call cpct_isKeyPressed_asm
    jp nz, input_arriba
    fin_arriba:

    ld hl, #Key_A                   ; Si es la tecla 'S' se mueve abajo
    call cpct_isKeyPressed_asm
    jp nz, input_abajo
    ld hl, #Joy0_Down               
    call cpct_isKeyPressed_asm
    jp nz, input_abajo
    fin_abajo:

    jp final_input
    
    input_derecha:                  ;; INPUT DERECHA
    ld a, #152
    ld d, Ent_W(ix)
    sla d
    sub d
    cp  Ent_X(ix)
    jp z, pararD                    ; Si la entidad no choca a la derecha 
        ld Ent_VX(ix), #1           ; VelocidadX = 1
        jp fin_derecha              
    pararD:                         ; Si choca
        ld Ent_VX(ix), #0           ; VelocidadX = 0 
        jp fin_derecha                 
    

    input_izquierda:                ;; INPUT IZQUIERDA
    ld a, #8
    cp Ent_X(ix)
    jp z, pararI                    ; Si la entidad no choca a la izquierda 
        ld Ent_VX(ix), #-1          ; VelocidadX = -1
        jp fin_izquierda             
    pararI:
        ld Ent_VX(ix), #0           ; Si choca
        jp fin_izquierda            ; VelocidadX = 0


    input_arriba:                   ;; INPUT ARRIBA
    ld a, #35
    cp Ent_Y(ix)
    jp z, pararAr                   ; Si la entidad no choca a arriba 
        ld Ent_VY(ix), #-1          ; VelocidadY = -1
        jp fin_arriba
    pararAr:
        ld Ent_VY(ix), #0           ; Si choca
        jp fin_arriba               ; VelocidadY = 0


    input_abajo:                    ;; INPUT ABAJO
    ld  a, #200 - HUD_SIZE_H
    sub Ent_H(ix)
    cp  Ent_Y(ix)
    jp  z, pararAb                  ; Si la entidad no choca la abajo 
        ld Ent_VY(ix), #1           ; VelocidadY = 1
        jp fin_abajo
    pararAb:
        ld Ent_VY(ix), #0           ; Si choca
        jp fin_abajo                ; VelocidadY = 0


    final_input:                    ;; FINAL DEL METODO

    call animaciones

    ld hl, #Key_Space               ; Si se pulsa la tecla 'ESPACIO'
    call cpct_isKeyPressed_asm
    jp nz, disparar
    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    jp nz, disparar
    jp saltar_disparo
    disparar:
        call man_game_player_shot
    saltar_disparo:

    pop de                          ; Mantenemos el registro 'DE'
jp input_finished

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Physics_Update_One_Entity'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT: (IX) Entidad a actualizar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Actualiza la posicion de la entidad
;; y si tiene el componente input se llama
;; a la funcion que lo controla
;; Destruye A
;;
sys_physics_update_one_entity::
    ld a, Ent_TYPE(ix)
    and #e_type_dead
    cp #e_type_dead
    jp nz, saltar_ret                   ; Si la entidad esta muerta no se actualiza su posicion
        ret
    saltar_ret:

    ld a, Ent_COMPONENTS(ix)            ; Si la entidad tiene el componente Input
    and #e_cmp_input
    cp #e_cmp_input
    jp z, sys_physics_check_keyboard    ; Llamamos a la funcion 'sys_physics_check_keyboard'
                                        ; antes de actualizar su posicion
    
    input_finished:

    ld a, Ent_Y(ix)
    add a, Ent_VY(ix)
    ld Ent_Y(ix), a                     ; Actualizamos la posicion en Y

    ld a, Ent_X(ix)
    add a, Ent_VX(ix)
    ld Ent_X(ix), a                     ; Actualizamos la posicion en X
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Physics_Update'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Llama a la funcion de entidad que 
;; recorre todo el array de entidades
;; para un componente en concreto (D)
;; y luego llama a la funcion en (HL)
;;
sys_physics_update::
    ld hl, #sys_physics_update_one_entity
    ld (forall_function), hl
    ld a, #e_cmp_movable
    ld (forall_filter), a                         ; Guardamos el componente en (D)
    call man_entity_forall_matching
ret