.module Render_System

.include "cpctelera.inc"
.include "cpctelera.h.s"
.include "entity.inc"
.include "sprites.inc"
.include "HUD.inc"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones Propias ;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl sys_render_one_entity
.globl man_entity_forall_matching
.globl sys_shifter_load
.globl sys_render_init_game

.globl cpct_setVideoMemoryPage_asm
.globl modulo

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Variables globales ;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl _main_palette

.globl cpct_drawSpriteMaskedAlignedTable_asm
.globl _g_masktable
.globl current_tilemap
redraw_size = MAX_ENTITIES*8

redraw_precalc: .ds redraw_size
.db 0xFF

next_render: .dw redraw_precalc

do_render:: .db 0
current_buffer:: .dw 0x8000
hat_timer:: .db 0
player_damaged:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;  Funciones ;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

sys_render_paint::
    loop_wait_paint:
        ld a, (do_render)
        cp #5
    jp nz, loop_wait_paint    
    
    ;ld a,#1                    ;PROTECCION CONTRA JU-
    modifier = .+1
    ld l, #0x20
    call cpct_setVideoMemoryPage_asm
    ld hl, #modifier
    ld a, #0x10
    xor (hl)
    ld (hl), a

    loop_wait_paint3:
        ld a, (do_render)
        cp #6
    jp nz, loop_wait_paint3
    loop_wait_paint4:
        ld a, (do_render)
        cp #1
    jp nz, loop_wait_paint4

    ld hl, #current_buffer
    inc hl
    ld a, #0x40
    xor (hl)
    ld (hl), a
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_Render_One_Entity'
;; REQUIERE EN IX DIRECCION DE LA ENTIDAD
;; DESTRUYE hl,bc
;;
;;
sys_render_redraw_group:
    ld hl, #redraw_precalc
    loop_erase:
    ld a, (hl)
    cp #0xFF
    jp z, saltar_erase
        inc hl
        inc hl
        ld d, (hl)
        inc hl
        ld e, (hl)
        ld a, #0x03
        inc hl
        inc hl
        inc hl
        ld b, (hl)
        inc hl
        ld c, (hl)
        push hl
        call cpct_drawSolidBox_asm      ;(2B DE) memory	Video memory pointer to the upper left box corner byte
                                        ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
                                        ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
                                        ;(1B B ) height	Box height in bytes (>0)
        pop hl
        inc hl
        jp loop_erase
    saltar_erase:

    ld hl, #redraw_precalc
    loop_redraw_group:
    ld a, (hl)
    cp #0xFF
    jp z, saltar_redraw_group
        ld d, a
        inc hl
        ld e, (hl)
        inc hl
        inc hl
        inc hl
        ld b, (hl)
        inc hl
        ld c, (hl)

        inc hl
        ld a, (hl)
        inc hl
        push hl
        ld l, (hl)
        ld h, a
        push hl
        pop ix

        ld h, #>_g_masktable
        ld l, #<_g_masktable
        call cpct_drawSpriteMaskedAlignedTable_asm
        pop hl
        inc hl
        jp loop_redraw_group
    saltar_redraw_group:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_Init'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el render
;;
sys_render_init::
    ld c, #0
    call cpct_setVideoMode_asm          ; Cambiamos el modo de video a 0 (16 colores)
    ld hl, #_main_palette
    ld de, #16
    call cpct_setPalette_asm            ; Aplicamos nuestra propia paleta de colores

    cpctm_setBorder_asm HW_BLACK        ; Cambiamos el color de los bordes

ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_init_game'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el render de la partida
;;
sys_render_init_game::
    ld de, (current_tilemap)
    call sys_render_tilemap
    call sys_render_init_HUD            ; Pintamos la zona del HUD
    call sys_shifter_load               ; Cargamos el shifter
ret
sys_render_borrar::
    ld e, Ent_PREVIO(ix)
    ld d, Ent_PREVIO2(ix)
    ld a, #0x03
    ld b, Ent_H(ix)
    ld c, Ent_W(ix)
    call cpct_drawSolidBox_asm
    ld e, Ent_PREVIO3(ix)
    ld d, Ent_PREVIO4(ix)
    ld a, #0x03
    ld b, Ent_H(ix)
    ld c, Ent_W(ix)
    call cpct_drawSolidBox_asm
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_Update'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Llama a la funcion de entidad que 
;; recorre todo el array en busca de dos
;; entidades con la componente de colision
;; y luego llama a la funcion en (HL)
;;
sys_render_one_entity::
    push de

    ld a, Ent_TYPE(ix)
    and #e_type_dead
    cp #e_type_dead                 ;Restamos pero sin modificar el registro
    jp z, saltar_borrar_muerto
        ld a, Ent_X(ix)             ;Recuperamos la X de la entidad
        and #1                      ;Sacamos el resto
        cp #1                       ;Comparamos con 1
        jp nz, saltar_shift
            ld de, (current_buffer)     ;Posicion inicial de la memoria de video
            ld c, Ent_X(ix)
            srl c                       ;Dividimos entre 2
            ld b, Ent_Y(ix)             ;Recuperamos la Y de la entidad
            call cpct_getScreenPtr_asm  ;LLamamos al getscreenPointer, devuelve el resultado en el registro HL
                                        ;Modifica los registros AF, BC, DE, HL
            ld d, Ent_SPRITE2(ix)
            ld e, Ent_SPRITE1(ix)
            inc de
            inc de
            jp saltar_salto
        saltar_shift:
            ld de, (current_buffer)              ;Posicion inicial de la memoria de video
            ld c, Ent_X(ix)
            srl c                       ;Dividimos entre 2
            ld b, Ent_Y(ix)             ;Recuperamos la Y de la entidad
            call cpct_getScreenPtr_asm  ;LLamamos al getscreenPointer, devuelve el resultado en el registro HL
                                        ;Modifica los registros AF, BC, DE, HL
            ld d, Ent_SPRITE2(ix)
            ld e, Ent_SPRITE1(ix)
        saltar_salto:
        ld b, h
        ld c, l

        ld hl, #next_render
        push hl

        ld a, (hl)
        inc hl
        ld h, (hl)
        ld l, a

        ld (hl), b
        inc hl
        ld (hl), c

        inc hl
        ld a, Ent_PREVIO4(ix)
        ld (hl), a
        inc hl
        ld a, Ent_PREVIO3(ix)
        ld (hl), a

        push de
        ld d,Ent_PREVIO2(ix)
        ld e,Ent_PREVIO(ix)
        ld Ent_PREVIO4(ix), d
        ld Ent_PREVIO3(ix), e
        pop de

        ld Ent_PREVIO2(ix), b
        ld Ent_PREVIO(ix), c

        inc de
        ld a, (de)
        inc hl
        ld (hl), a
        dec de
        inc hl
        ld a, (de)
        ld (hl), a

        inc hl
        ld a, Ent_H(ix)
        ld (hl), a
        inc hl
        ld a, Ent_W(ix)
        ld (hl), a
        inc hl

        pop de
        ld a, l
        ld (de), a
        inc de
        ld a, h
        ld (de), a
    jp saltar_sys_render
    
    saltar_borrar_muerto::
        call sys_render_borrar
    saltar_sys_render::
    pop de
ret
clear_render_erase::
    ld hl, #redraw_precalc
    ld (hl), #0xFF
    ld de, #redraw_precalc+1
    ld bc, #redraw_size-1
    ldir 
    ld hl, #redraw_precalc
    ld (next_render), hl
ret
sys_render_update::
    ld hl, #sys_render_one_entity
    ld (forall_function), hl
    ld a,#e_cmp_render
    ld (forall_filter), a
    call man_entity_forall_matching
    call sys_render_redraw_group    ;ERASE-DRAW BUFFER
    ld hl, #sys_render_HUD                  ; Puntero a funcion
    call man_HUD_forall           
    call sys_render_paint           ;SWITCH BUFFER
    call clear_render_erase
    call blink_hat
ret

.globl _tiles_00
.globl _tilemap_01
.globl _tilemap_02

;requiere en de tilemap
sys_render_tilemap::
    ;;(1B C) width	Width in tiles of the view window to be drawn
    ;;(1B B) height	Height in tiles of the view window to be drawn
    ;;(2B DE) tilemapWidth	Width in tiles of the complete tilemap
    ;;(2B HL) tileset	Pointer to the start of the tileset definition (list of 32-byte tiles).
    push de

    ld bc, #0x1914
    ld de, #0x14
    ld hl, #_tiles_00
    call cpct_etm_setDrawTilemap4x8_ag_asm

    pop de
    push de
    ld hl, #0xC000
    call cpct_etm_drawTilemap4x8_ag_asm
    pop de
    ld hl, #0x8000
    call cpct_etm_drawTilemap4x8_ag_asm
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;   FUNCIONES DEL HUD    ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_init_HUD'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa el dibujado del HUD
;;
sys_render_init_HUD::
    call sys_render_border_HUD
    call sys_render_letras_HUD
    ld   bc, #_spr_heart
    call sys_render_heart
    call clear_render_erase
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_heart'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Dibuja el sprite del corazon
sys_render_heart::
    push bc
    ld hl, #0xc000
    call render_heart_in_page
    pop bc
    ld hl, #0x8000
    call render_heart_in_page
ret
render_heart_in_page:
    ld    e, #SPR_HEART_W
    ld    d, #SPR_HEART_H
    push de
    pop ix
    ld d,h
    ld e,l
    ld hl, #10070
    add hl, de
    ld d,h
    ld e,l
    ld h, #>_g_masktable
    ld l, #<_g_masktable
    call cpct_drawSpriteMaskedAlignedTable_asm
ret
blink_hat:
    ld a,(hat_timer)
    cp #0
    jp z,finish_blink
        dec a
        ld (hat_timer),a
        jp no_finish_blink
    finish_blink:
        ld ix,(direccion_jugador)
        cpctm_setBorder_asm HW_BLACK        ; Cambiamos el color de los bordes
        ld a,Ent_COMPONENTS(ix)
        or #e_cmp_render
        ld Ent_COMPONENTS(ix),a
        ld a,#0
        ld (player_damaged),a
    no_finish_blink:
    ld a,(player_damaged)
    cp #1
    jp nz,no_parpadear_jugador
        ld a,(hat_timer)
        and #1
        cp #1
        jp nz,no_parpadear_jugador
            ld ix,(direccion_jugador)
            call blink_render
    no_parpadear_jugador:
ret
blink_render::
    call sys_render_borrar
    blink_render_enemy::
    ld a,Ent_COMPONENTS(ix)
    xor #e_cmp_render
    ld Ent_COMPONENTS(ix),a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_HUD'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INPUT:   (IY) Valor a dibujar
;;           (B) Valor de la posicion X
;;           (C) Valor de la posicion Y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Dibuja el número que se le pasa por 
;; parámetro en la posicion
;; Destruye HL, DE
sys_render_HUD::
    ld   de, (current_buffer) ;; Puntero al principio de la memoria
    push bc
    call cpct_getScreenPtr_asm    ;; Calcula la direccion de memoria a pintar y la devuelve en HL
    pop  bc
    push bc
    push ix
    call _myDrawStringM0          ;; Draw the string
    pop  ix
    pop  bc
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_border_HUD'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Dibuja los border del HUD
;;
sys_render_border_HUD::
    ld hl, #0xc000
    call render_border_hud_in_buffer
    ld hl, #0x8000
    call render_border_hud_in_buffer    
ret
render_border_hud_in_buffer:
    ;; LINEA SUPERIOR
    push hl
    ld bc,#1841
    call render_border_hud_in_buffer_line
    ;; LINEA INFERIOR
    pop hl
    ld bc,#16257
    call render_border_hud_in_buffer_line
ret
render_border_hud_in_buffer_line:
    push bc
    push hl
    add hl, bc
    ld d,h
    ld e,l
    pop hl
    pop bc
    dec bc
    add hl, bc 
    ld   bc, #79        ; Pinta 79 pixeles
    ld    a, #0x3F      ; Color de la linea
    ld (hl), a  
    LDIR                ; Pintamos una primera linea horizontal
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Funcion 'sys_render_letras_HUD'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; No tiene INPUT
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Dibuja las letras del HUD
;;
stringScore:  .asciz "SC"
stringLv: .asciz "LV"
stringBar: .asciz "-"
.globl _myDrawStringM0
sys_render_letras_HUD::
   ld de, #0x8000
   call render_letras_HUD_in_buffer
   ld de, #0xc000
   call render_letras_HUD_in_buffer
ret
render_letras_HUD_in_buffer:
    push de
    ld    h, #04                 ;; Color de Fondo
    ld    l, #15                  ;; Color de las letras
    call cpct_setDrawCharM0_asm   ;; Set draw char colours

    pop de
    push de
    ld    b, #188                 ;; Coordenada Y
    ld    c, #49                  ;; Coordenada X
    call cpct_getScreenPtr_asm    ;; Calcula la direccion de memoria a pintar y la devuelve en HL
    ld   iy, #stringScore         ;; Puntero al string
    call _myDrawStringM0          ;; Draw the string

    pop de
    push de
    ld    b, #188                 ;; Coordenada Y
    ld    c, #1                   ;; Coordenada X
    call cpct_getScreenPtr_asm    ;; Calcula la direccion de memoria a pintar y la devuelve en HL
    ld   iy, #stringLv             ;; Puntero al string
    call _myDrawStringM0          ;; Draw the string

    pop de
    ld    b, #188                 ;; Coordenada Y
    ld    c, #15                  ;; Coordenada X
    call cpct_getScreenPtr_asm    ;; Calcula la direccion de memoria a pintar y la devuelve en HL
    ld   iy, #stringBar             ;; Puntero al string
    call _myDrawStringM0          ;; Draw the string
ret